@@ -6,7 +6,7 @@
 # the same terms of the Ruby's license.
 #
 #
-# $Id: server_state.rb 181 2007-02-20 15:39:21Z znz $
+# $Id$
 # Create : K.S. 04/04/20 10:42:27
 #
 
@@ -195,7 +195,7 @@ module Nadoka
     def on_join user, rch
       ch = canonical_channel_name(rch)
       if user == nick
-        chs = @current_channels[ch] = ChannelState.new(rch)
+        @current_channels[ch] = ChannelState.new(rch)
       else
         if @current_channels.has_key? ch
           @current_channels[ch].on_join(user)
@@ -313,7 +313,9 @@ module Nadoka
     # ex) :NadokaProgram 403 simm !hoge :No such channel
     def on_403 ch
       if safe_channel?(ch) && ch[1] != ?!
-        @manager.join_to_channel( "!" + ch)
+        if @config.channel_info[ch] && @config.channel_info[ch][:auto_create]
+          @manager.join_to_channel( "!" + ch)
+        end
       end
     end
     
