@@ -12,7 +12,7 @@ Original Credit:
 == Modified
 
   Modified by K.Sasada.
-  $Id: irc.rb 250 2011-06-01 22:51:17Z znz $
+  $Id$
   
 =end
 
@@ -58,8 +58,8 @@ module RICE
 
       @read_q  = Queue.new
 
-      @read_th  = Thread.new(@read_q, @eol) do |read_q, eol|
-        read_thread(read_q, eol)
+      @read_th  = Thread.new(@read_q, @eol) do |read_q, eol_|
+        read_thread(read_q, eol_)
       end
 
       @threads = {}
@@ -86,7 +86,7 @@ module RICE
       @delay = 0.3
       @prev_send_time = Time.now
     end
-    attr :delay, true
+    attr_accessor :delay
     attr_reader :server, :port, :ssl_params
     
 =begin
@@ -200,9 +200,11 @@ module RICE
         end
         sleep retry_wait
         retry
-      ensure
+      rescue Exception
+        @main_th.kill
+        raise
+      else
         @main_th.join
-        nil
       end
 
     end
@@ -215,6 +217,7 @@ module RICE
           context.set_params(ssl_params)
           conn = OpenSSL::SSL::SSLSocket.new(conn, context)
           conn.sync_close = true
+          conn.hostname = @server if conn.respond_to? :hostname=
           conn.connect
           if context.verify_mode != OpenSSL::SSL::VERIFY_NONE
             conn.post_connection_check(@server)
@@ -244,8 +247,8 @@ module RICE
     USER_THREAD = Struct.new('User_Thread', :q, :raise_on_close)
     def regist(raise_on_close = false, *args)
       read_q = Queue.new
-      th = Thread.new(read_q, self, *args) do |read_q, conn, *args|
-        yield(read_q, conn, *args)
+      th = Thread.new(read_q, self, *args) do |read_q_, conn, *args_|
+        yield(read_q_, conn, *args_)
       end
       @threads.synchronize do
         @threads[th] = USER_THREAD.new(read_q, raise_on_close)
@@ -298,7 +301,7 @@ module RICE
         
       rescue Closed
         begin
-          @main_th.run if @main_th.alive?
+          @main_th.run if alive?
         rescue Closed
         end
         retry
@@ -315,7 +318,7 @@ module RICE
     def close(restart = false)
       begin
         unless restart
-          @main_th.exit if @main_th.alive?
+          @main_th.exit if alive?
           @read_th.exit if @read_th.alive?
         end
         
@@ -555,15 +558,17 @@ module RICE
       str << @command
 
       if @params
-        f = false
         @params.each do |param|
           str << ' '
-          if (param == @params[-1]) && (param.size == 0 || /(^:)|(\s)/ =~ param)
+          param_s = param.to_s
+          if param_s.respond_to?(:force_encoding)
+            param_s = param_s.dup.force_encoding(Encoding::ASCII_8BIT)
+          end
+          if (param == @params[-1]) && (param_s.size == 0 || /\A:|\s/ =~ param_s)
             str << ':'
-            str << param.to_s
-            f = true
+            str << param_s
           else
-            str << param.to_s
+            str << param_s
           end
         end
       end
@@ -658,7 +663,13 @@ E
         str << @params[0]
 
         str << ' :'
-        str << @params[1..-1].join(' ')
+        if "".respond_to?(:force_encoding)
+          str << @params[1..-1].map{|param|
+            param.force_encoding(Encoding::ASCII_8BIT)
+          }.join(' ')
+        else
+          str << @params[1..-1].join(' ')
+        end
 
         str << "\x0D\x0A"
         str
@@ -695,7 +706,11 @@ E
        003,RPL_CREATED
        004,RPL_MYINFO
        005,RPL_BOUNCE
+       010,RPL_REDIR
+       015,RPL_MAP 016,RPL_MAPMORE 017,RPL_MAPEND 018,RPL_MAPSTART
+       020,RPL_HELLO 042,RPL_YOURID 043,RPL_SAVENICK
        302,RPL_USERHOST 303,RPL_ISON 301,RPL_AWAY
+       304,RPL_TEXT
        305,RPL_UNAWAY 306,RPL_NOWAWAY 311,RPL_WHOISUSER
        312,RPL_WHOISSERVER 313,RPL_WHOISOPERATOR
        317,RPL_WHOISIDLE 318,RPL_ENDOFWHOIS
@@ -703,7 +718,9 @@ E
        369,RPL_ENDOFWHOWAS 321,RPL_LISTSTART
        322,RPL_LIST 323,RPL_LISTEND 325,RPL_UNIQOPIS
        324,RPL_CHANNELMODEIS 331,RPL_NOTOPIC
-       332,RPL_TOPIC 341,RPL_INVITING 342,RPL_SUMMONING
+       332,RPL_TOPIC 333,RPL_TOPICWHOTIME
+       341,RPL_INVITING 342,RPL_SUMMONING
+       344,RPL_REOPLIST 345,RPL_ENDOFREOPLIST
        346,RPL_INVITELIST 347,RPL_ENDOFINVITELIST
        348,RPL_EXCEPTLIST 349,RPL_ENDOFEXCEPTLIST
        351,RPL_VERSION 352,RPL_WHOREPLY 315,RPL_ENDOFWHO
@@ -724,6 +741,7 @@ E
        252,RPL_LUSEROP 253,RPL_LUSERUNKNOWN 254,RPL_LUSERCHANNELS
        255,RPL_LUSERME 256,RPL_ADMINME 257,RPL_ADMINLOC1
        258,RPL_ADMINLOC2 259,RPL_ADMINEMAIL 263,RPL_TRYAGAIN
+       265,RPL_LOCALUSERS 266,RPL_GLOBALUSERS
        401,ERR_NOSUCHNICK 402,ERR_NOSUCHSERVER 403,ERR_NOSUCHCHANNEL
        404,ERR_CANNOTSENDTOCHAN 405,ERR_TOOMANYCHANNELS
        406,ERR_WASNOSUCHNICK 407,ERR_TOOMANYTARGETS
